from datetime import datetime
from sanic import Sanic, response
import uvloop, asyncio
from motor.motor_asyncio import AsyncIOMotorClient
import pymongo
import hashlib
import jinja2
import json
import os
import smtplib
import const

app = Sanic(__name__)
asyncio.set_event_loop_policy(uvloop.EventLoopPolicy()) # swap default event loop to uvloop

#mongodb = MongoClient('mongodb://172.21.27.1,172.21.27.2,172.21.27.3:27017/', replicaSet="kspace-mongo-set").kspace_accounting
mongodb = AsyncIOMotorClient(const.MONGO_URI)
mongodb = mongodb.get_default_database()

DOORBOY_SECRET = os.environ["DOORBOY_SECRET"]

assert len(DOORBOY_SECRET) > 10

@app.route("/allowed")
async def view_doorboy_uids(request):
    if request.headers.get('KEY') != DOORBOY_SECRET:
        return "how about no"
    allowed_names = [o["_id"] for o in await mongodb.member.find({"enabled": True}).to_list()]
    allowed_uids = []
    for obj in await mongodb.inventory.find({"token.uid_hash": {"$exists":True}, "inventory.owner_id": {"$exists":True}, "token.enabled": True}, {"inventory.owner_id": True, "token.uid_hash": True }):
        if obj["inventory"].pop("owner_id") in allowed_names:
            del obj["_id"]
            del obj["inventory"]
            allowed_uids.append(obj)
    return response.json(allowed_uids=allowed_uids)


@app.route("/longpoll")
async def view_longpoll(request):
    if request.headers.get('KEY') != DOORBOY_SECRET:
        return "how about no"

    async def g():
        yield "data: response-generator-started\n\n"
        pipeline = [
            {
                '$match': {
                     'operationType': "insert",
#                     'component': 'doorboy',
#                     'type': 'open-door'
                 }
            }
        ]
        try:
            async with mongodb.eventlog.watch(pipeline) as stream:
                yield "data: watch-stream-opened\n\n"
                async for event in stream:
                    if event["fullDocument"].get("type") == "open-door":
                        yield "data: %s\n\n" % event["fullDocument"]["door"]
        except pymongo.errors.PyMongoError:
            return "urror"
    return stream(g, content_type="text/event-stream")

if __name__ == '__main__':
    app.run(debug=False, host='::')
